use strict;
use warnings;
use Module::Build;

my $builder = Module::Build->new(
    module_name         => 'LaTeX::Decode',
    license             => 'perl',
    dist_author         => q{François Charette <firmicus@cpan.org>},
    dist_version_from   => 'lib/LaTeX/Decode.pm',
    build_requires => {
        'Test::More' => 0,
    },
    requires => {
        'perl' => 5.008,
        'File::Slurp' => 0,
        'Unicode::Normalize' => 0,
    },
    script_files => [ glob( "bin/*" ) ],
    add_to_cleanup      => [ 'LaTeX-Decode-*' ],
#    create_makefile_pl => 'traditional',
);

$builder->create_build_script();
